
function [dtw,low_partcol,high_partcol,low_partcol_err,high_partcol_err]=...
    TARDISS_nc_CO(etl,outputfn,zobs,localtime)

% function TARDISS_nc_CO
% function to infer partial column values from the total column DMF values
% reported by TCCON in the private.nc files

% Input:
% etl - matlab structure containing the variables and data in the
% private.nc files reported by TCCON. The files can be reduced to include
% only the necessary variables from the private.nc file.
%
% outputfn - your chosen output file name as a string. 'output_etl' for
% example.
%
% zobs - observational elevation of the TCCON site in km. 
%
% localtime - standard time offset from UTC time. This is needed to align
% measurements to be within the same numerical day to avoid fitting values
% between days. 

% Output:
% ML - structure containing all the fit values described at the bottom of
% the script when they are assigned. ML is normally not saved in order to
% save space. Add to the output variables and comment line 575 to save.
%
% dtw - datenum vector for time series
% 
% low_partcol - lower partial column DMF values in ppb
% 
% high_partcol - upper partial column DMF values in ppb
% 
% low_partcol_err - lower partial column DMF error values in ppb
% 
% high_partcol_err - upper partial column DMF error values in ppb
% 
%
% Collated vectors of retrieved values are described at the end of the
% script and contain the values from the matrices in ML in vector format
% for ease of plotting. 

flag=etl.flag; % assign qc flag vector

% Operational fitting parameters - do not change unless testing 
sc=3;
xp=3;


zminall=etl.zmin;%assign larger zmin vector

mavtime=etl.prior_time; %assign prior times
mavdtv=datevec(mavtime-(localtime/24));%adjust time to local standard time


o2vsf=etl.vsf_o2; %assign O2 VSFs

%load in averaging kernels for the CO2 windows
load('coakmats_TARDISS.mat')

%interpolating the aks to sza degrees
akbin=[10:10:90];
akbinint=[10:90];

for ii=1:51
%     akv2158(ii,:)=interp1([10:10:90],co2158akmat(ii,:),[10:90]);
    akv2160(ii,:)=interp1([10:10:90],co2160akmat(ii,:),[10:90]);
    akv2111(ii,:)=interp1([10:10:90],co2111akmat(ii,:),[10:90]);
    akv4290(ii,:)=interp1([10:10:90],co4290akmat(ii,:),[10:90]);
end

% co2158akmat=akv2158;
co2160akmat=akv2160;
co2111akmat=akv2111;
co4290akmat=akv4290;


%change to pst and get unique days
etl.utc=etl.time;
etl.dtw=etl.time-(localtime/24);
days=unique(floor(etl.dtw));
days=days(days>0);


%round sza to nearest 1 degree
szabin=round(etl.solzen);
diffzen=etl.solzen-szabin;
 ll=length(szabin);
bin=[min(szabin):1:max(szabin)];
count=0;
for ii=bin
    count=count+1;
    etl.bin(szabin==ii,1)=count;
end

dayleng=length(days);%get number of unique days

% QC flag filter with additional conditions for mir windows
zfilt=(flag==0&etl.vsw_ada_xco_2111_insb<1e34&etl.vsw_ada_xco_2160_insb<1e34...
    &etl.vsw_co_2111_error_insb<1e34&etl.vsw_co_2160_error_insb<1e34);

jjcount=0;%start count for days of measurement

for jj=1:dayleng
    
    jjcount=jjcount+1;
     
    clear aaa aaaa

    mask=(floor(etl.dtw)==days(jj))&(zfilt==1);%create filter for the day and the QC filter
    if sum(mask)==0
        jjcount=jjcount-1;
        continue
    end
    
     %assign daily values for variables and align a priori profiles
    asza=etl.solzen(mask);
    daynum=sum(mask);
    mm=(daynum-1)./2+1;
    mn=([1:daynum]'-mm).^2;
    mn=mn./max(mn);

    dayvec=datevec(etl.dtw(mask));
    monvec=unique(dayvec(:,2));
    dowvec=unique(weekday(etl.dtw(mask)));
    datw=etl.dtw(mask);
    datv=datevec(datw);
    dayun=unique(datv(:,1:3),'rows');
    lldatw=length(datw);
    
    zmin=zminall(mask);
    hite=etl.ak_altitude;
    luft=etl.xluft(mask);
    
    mavmask=sum(mavdtv(:,1:3)==dayun,2)==3;
    
    mavday=mavdtv(mavmask,:);
    mavdatw=datenum(mavday);
    llmavdatw=length(mavdatw);

    bb=length(bin);
    %find the nearest prior profile to the current point in the iteration
    for ii=1:llmavdatw
        aaa(:,ii)=(datw-mavdatw(ii)).*24;
        aaaa(:,ii)=abs(aaa(:,ii))<1.50;
    end
    
    %assign matmav values
    coap = etl.prior_1co(:,mavmask);
    denap = etl.prior_density(:,mavmask);
    pres = etl.prior_pressure(:,mavmask);
    hiteapdt = (hite'.*ones(lldatw,51));
    oxygen=etl.prior_0o2(:,mavmask);
    water=etl.prior_1h2o(:,mavmask);
    coapdt=zeros(lldatw,51);
    presapdt=zeros(lldatw,51);
    denapdt=zeros(lldatw,51);
    oxygenapdt=zeros(lldatw,51);
    waterapdt=zeros(lldatw,51);
    
   
   % determine the partial column split point and the ratio of the levels
   % given the partial column split for integration
    lowerlim=nanmin(find(hite(:,1)-zobs>0));
    upperlim=nanmin(find(hite(:,1)-zobs-2>0));
    upperpathratio=((zobs+2)-hite(upperlim-1,1))./...
        (hite(upperlim,1)-hite(upperlim-1,1));
    
    apscale=ones(lldatw,1);
    
    com2160=etl.vsw_ada_xco_2160_insb(mask);
    com2111=etl.vsw_ada_xco_2111_insb(mask);
    con4290=etl.vsw_ada_xco_4290(mask);
    
    comerr2160=(etl.vsw_co_2160_error_insb(mask)./etl.vsw_co_2160_insb(mask)).*com2160; 
    comerr2111=(etl.vsw_co_2111_error_insb(mask)./etl.vsw_co_2111_insb(mask)).*com2111; 
    conerr4290=(etl.vsw_co_4290_error(mask)./etl.vsw_co_4290(mask)).*con4290; 

    
    for zz=1:llmavdatw
        for kk=1:lldatw
            if aaaa(kk,zz)==1  
                coapdt(kk,:)=coap(:,zz);
                presapdt(kk,:)=pres(:,zz);
                denapdt(kk,:)=denap(:,zz);
                oxygenapdt(kk,:)=oxygen(:,zz); 
                waterapdt(kk,:)=water(:,zz);
            end  
        end
    end
    
    % calculating the oxygen total column data
    oxygenapdt=(oxygenapdt.*o2vsf(mask));
    
    % calculate original vertical columns of species and oxygen
    [~,co_ovc,~]=vert_int(denapdt,hiteapdt,presapdt,coapdt,5,zmin,lowerlim,zobs);
    [~,ox_ovc,~]=vert_int(denapdt,hiteapdt,presapdt,oxygenapdt,5,zmin,lowerlim,zobs);
    
    coovc=co_ovc'./(ox_ovc./0.2095)';
    
    % calculate the median total column scalar 
    apscaleyy=nanmedian([com2160,com2111,con4290],2);
    
    apscale=apscaleyy./coovc;

        
    %Make the jacobian values - termed k values for K matrix in the paper
    
    Jmir2160=zeros(lldatw,51);
    Jmir2111=zeros(lldatw,51);
    Jnir4290=Jmir2111;
    
    daybin=etl.bin(mask);
    
    for kk=1:daynum
        Jmir2160(kk,:)=co2160akmat(:,daybin(kk))';
        Jmir2111(kk,:)=co2111akmat(:,daybin(kk))';
        Jnir4290(kk,:)=co4290akmat(:,daybin(kk))';
    end
     
     coapdtfull=coapdt.*apscale;
    
        %recreating the scaled ovcs
    coapdtfull2160=coapdtfull;
    coapdtfull2111=coapdtfull;
    coapdtfull4290=coapdtfull;

    %assign multipliers alone since retrieving column values
    %%

    
    %assign multipliers with background prior
        
        
    jmco2160full=Jmir2160.*coapdtfull2160;
    jmco2111full=Jmir2111.*coapdtfull2111;
    jnco4290full=Jnir4290.*coapdtfull4290;
        
     %integrate to get the jacobian values
    [calparcolox,calccolox,calparcoloxup]=...
        vert_int(denapdt,hiteapdt,presapdt,oxygenapdt,5,zmin,lowerlim,zobs);
    
    [calparcolj2160,~,calparcolupj2160]=...
        vert_int(denapdt,hiteapdt,presapdt,jmco2160full,5,zmin,lowerlim,zobs);
    
    [calparcolj2111,~,calparcolupj2111]=...
        vert_int(denapdt,hiteapdt,presapdt,jmco2111full,5,zmin,lowerlim,zobs);
    
    [calparcolj4290,~,calparcolupj4290]=...
        vert_int(denapdt,hiteapdt,presapdt,jnco4290full,5,zmin,lowerlim,zobs);
   
    JLamir2160=calparcolj2160.*(0.2095./calccolox');
    JLamir2111=calparcolj2111.*(0.2095./calccolox');
    JLanir4290=calparcolj4290.*(0.2095./calccolox');
    
    JUmir2160=calparcolupj2160.*(0.2095./calccolox');
    JUmir2111=calparcolupj2111.*(0.2095./calccolox');
    JUnir4290=calparcolupj4290.*(0.2095./calccolox');

    %create the jacobian matrix
    Gw=[diag(JLamir2160), diag(JUmir2160);...
        diag(JLamir2111), diag(JUmir2111);...
        diag(JLanir4290), diag(JUnir4290)];
    
        if sum(sum(isnan(Gw)))~=0
            jjcount=jjcount-1;
            continue
        end
        
    %set number of windows for multipliers
    numwin=3;
    
    
    %create Y vector, do in situ correction, and calculate error values
    %Order matters for the alignment of the vectors
     yy=[com2160;com2111;con4290];
    yyerr=[comerr2160;comerr2111;conerr4290];
    
    apscaleyy=nanmedian([com2160,com2111,con4290],2);
    
    %original uniform apscale
    oxscalevec=repmat(calccolox',numwin,1)./0.2095;
    
    [calcparcol,altovc2160,calcparcolup]=...
        vert_int(denapdt,hiteapdt,presapdt,coapdtfull2160,5,zmin,lowerlim,zobs);
    
    [~,altovc2111,~]=...
        vert_int(denapdt,hiteapdt,presapdt,coapdtfull2111,5,zmin,lowerlim,zobs);
    
    [~,altovc4290,~]=...
        vert_int(denapdt,hiteapdt,presapdt,coapdtfull4290,5,zmin,lowerlim,zobs);
 
    ovcvec=[altovc2160';altovc2111';altovc4290']./oxscalevec;
    xaovc=ovcvec;
    
    apscalevec=repmat(apscaleyy,numwin,1);
    
    %create Se, model covariance matrix
    W=diag(yyerr).^2;
   %%
   
   if sc==1
    % % five
    nn=1e-5;
    mm=1e-5;
    
   elseif sc==2
    % % five five
    nn=5e-5;
    mm=5e-5;
    
   elseif sc==3
    % % four
    nn=1e-4;
    mm=1e-4;
   end
    
    %create Sa
    Sa=[diag(mm.*ones(daynum,1)),diag(zeros(daynum,1));...
        diag(zeros(daynum,1)),diag(nn.*ones(daynum,1))];


    %create off-diagonals for Sa
   %%{
    lim=floor(daynum)/3; %original , increasing 3 shortens covariance time
    for indi=1:lim
        for ind = 1:daynum-indi
            Sa(daynum+ind,daynum+ind+indi)=nn.*exp(-(indi+1)/(lim/2));
            Sa(daynum+ind+indi,daynum+ind)=nn.*exp(-(indi+1)/(lim/2));
        end
    end
     %}
    
    %reassign values to match Rodgers/Connor
    Se=W;
    K=Gw;


    %create vectors and matrices for calculation
 d = (yy-xaovc);
 G=K;
 RG=rank(G);
 full=RG==2*daynum;

 %calculating the least squares solution and errors for least squares
    %solution
    
 ml2=inv(G'*G)*G'*d; %least squares solution
 
  Fx=G*ml2;
  
  nel=length(Fx)./3;
  RSS=zeros(nel,1);
  for ii=1:(nel)
      RSS(ii,1)=((Fx(ii,1)-d(ii,1)).^2)+((Fx(ii+nel,1)-d(ii+nel,1)).^2)+...
          ((Fx(ii+2*nel,1)-d(ii+2*nel,1)).^2);
  end
  RSS=[RSS;RSS];
  
  Cov=inv(G'*G);
  Var=RSS.*Cov;
  ml2err=sqrt(diag(Var));%least squares errors

 y=(K*(ml2))+xaovc;%result of forward model with least squares solution - unused
    
    %%
    %assigning the a priori partial column scalar values
    
 if xp==1
 % % % median prior - mp
 xa=[1.*nanmedian(ml2(1:daynum,1)).*ones(daynum,1);...
     1.*nanmedian(ml2(daynum+1:end,1)).*ones(daynum,1)];
 
 elseif xp==2
 % % L2 prior - lp 
  xa=[1.*(ml2(1:daynum,1));ml2(daynum+1:end,1)];

 elseif xp==3
% % static prior - sp - use this one for operational co retrievals
 xa=[zeros(daynum,1);zeros(daynum,1)];
 end

%calculate the maximum a posteriori solution
 map=xa+Sa*G'*inv(G*Sa*G'+Se)*(d-G*xa);
 
 ymap=(K*(map))+xaovc;%result of forward model with MAP solution - unused
 
 
 %fisher information matrix
 fishinfo=G'*inv(Se)*G+inv(Sa);
 %Averaging kernel
 Amat=inv(G'*inv(Se)*G+inv(Sa))*G'*inv(Se)*G;
 %calculate degrees of freedom in the fit
 ds=trace(Amat);%total DoF

 dslow=trace(Amat(1:daynum,1:daynum));% lower partial column DoF
 dshigh=trace(Amat(daynum+1:end,daynum+1:end));% upper partial column DoF
  
 % calculate shannon information matrix
 H=-0.5.*log(det(eye(length(Amat))-Amat));

 %covariance matrix
 covmat=inv(fishinfo);
 % calculate error of MAP retrieval
 stderr=zeros(length(Amat),1);
 for sstd=1:length(Amat)
    stderr(sstd)=sqrt(abs(covmat(sstd,sstd)));
 end
% calculate the retrieval noise and smoothing error
 s_hat=inv(G'*inv(Se)*G+inv(Sa));
 part=inv(G'*inv(Se)*G+inv(Sa));
Ssp=part'*inv(Sa)*part;
Srp=part'*G'*inv(Se)*G*part;
Ss=diag(Ssp);
Sr=diag(Srp);
 
% check for unphysical retrieval values
sealter=inv(Se.^0.5);
saalter=Sa.^0.5;
Khat=sealter*K*saalter;
    if sum(sum(isnan(Khat)))>0
        jjcount=jjcount-1;
        continue
    end
[Uk,Sk,Vk] = svd(Khat);
minkhat=min(size(Khat));
for ii=1:minkhat
sdiag(ii,1)=Sk(ii,ii);
end
sdiaggtone=sum(sdiag>1);
  
% Calculate further characterization data
calccol=calcparcolup;
 [calcparcol_ox,totoxcol,calccol_ox,oxycol]=vert_int(denapdt,hiteapdt,presapdt,oxygenapdt,5,zmin,lowerlim,zobs);
[~,~,~,watercol]=vert_int(denapdt,hiteapdt,presapdt,waterapdt,5,zmin,lowerlim,zobs);
 

    %calculate gain matrix and vertical sensitivity values
gain=inv(G'*inv(Se)*G+inv(Sa))*G'*inv(Se);
    
jmat=[Jmir2160;Jmir2111;Jnir4290];

scalarprofile= gain*(jmat).*([coapdtfull.*apscale;coapdtfull.*apscale]);

lowak=(scalarprofile(1:daynum,:));
highak=(scalarprofile(daynum+1:end,:));

ML{1,jj}=ml2(1:daynum)+1;
ML{2,jj}=ml2(daynum+1:end)+1;
ML{3,jj}=calcparcol;
ML{4,jj}=datw;
ML{5,jj}=full;
ML{7,jj}=yy;
ML{8,jj}=y;
ML{9,jj}=map(1:daynum)+1;
ML{10,jj}=map(daynum+1:end)+1;
ML{11,jj}=ymap;
ML{14,jj}=calcparcolup;
ML{15,jj}=calcparcol_ox;
ML{17,jj}=H;
ML{24,jj}=daynum;
ML{25,jj}=calccol;
ML{26,jj}=calparcoloxup;
ML{27,jj}=etl.solzen(mask);
ML{29,jj}=etl.tout(mask);
ML{30,jj}=stderr(1:daynum);
ML{31,jj}=stderr(daynum+1:end);
ML{38,jj}=dslow;
ML{39,jj}=dshigh;
ML{43,jj}=ds;

ML{81,jj}=Ss(1:daynum);
ML{82,jj}=Sr(1:daynum);
ML{83,jj}=Ss(daynum+1:end);
ML{84,jj}=Sr(daynum+1:end);

ML{89,jj}=totoxcol;

ML{97,jj}=etl.xluft(mask);

end

%% creating full vectors from the daily vectors from the retrieval

[~,dayleng]=(size(ML));

%create empty space for values
ls=[];% least squares lower partial column scalar
us=[];% least squares upper partial column scalar
lowercon=[];% a priori species lower partial column value
uppercon=[];% a priori species upper partial column value
RG=[];% specification if the retrieval is full rank - unused
lsmap=[];% MAP lower partial column scalar
usmap=[];% MAP upper partial column scalar
dtw=[];% datenum time values for the day
tccon=[];% measurement vector for fit
mappost=[];% forward model solution from MAP solution
mlpost=[];% forward model solution from least squares solution
oxcon=[];% a priori O2 lower partial column value
shannon=[];% Shannon information content
ptsinday=[];% number of measurements in a day
colox=[];% a priori O2 total column value
colmol=[];% a priori O2 upper partial column value
solzen=[];% solar zenith angles for the day
temperature=[];% ground temperature measured by TCCON
lowerr=[];% MAP lower partial column error values
uperr=[];% MAP upper partial column error values
dsl=[];% lower partial column daily DoFs
dsh=[];% upper partial column daily DoFs
totalds=[];% total daily partial column daily DoFs
smterr=[];% lower partial column smoothing error
rtrerr=[];% lower partial column retrieval noise
smterrup=[];% upper partial column smoothing error
rtrerrup=[];% upper partial column retrieval noise
oxtotcol=[];% total oxygen column value in molec/cm^2
luftluft=[];%xluft vector


%iterate over the retrieval days
for ii= 1:dayleng
    
    ls=[ls;ML{1,ii}];
    us=[us;ML{2,ii}];
    lowercon=[lowercon;ML{3,ii}];
    RG=[RG;ML{5,ii}];
    lsmap=[lsmap;ML{9,ii}];
    usmap=[usmap;ML{10,ii}];
    dtw=[dtw;ML{4,ii}];
    tccon=[tccon;ML{7,ii}];
    mappost=[mappost;ML{11,ii}];
    mlpost=[mlpost;ML{8,ii}];
    uppercon=[uppercon;ML{14,ii}];
    oxcon=[oxcon;ML{15,ii}];
    if ~isinf(ML{17,ii})
        shannon=[shannon;ML{17,ii}];
    else
        shannon=[shannon;zeros(length(ML{17,ii}))];
    end
    ptsinday=[ptsinday;ML{24,ii}];
    colmol=[colmol;ML{25,ii}];
    colox=[colox;ML{26,ii}];
    solzen=[solzen;ML{27,ii}];
    temperature=[temperature;ML{29,ii}];
    lowerr=[lowerr;ML{30,ii}];
    uperr=[uperr;ML{31,ii}];
    dsl=[dsl;ML{38,ii}];
    dsh=[dsh;ML{39,ii}];
    totalds=[totalds;ML{43,ii}];
    smterr=[smterr;ML{81,ii}];
    rtrerr=[rtrerr;ML{82,ii}];
    smterrup=[smterrup;ML{83,ii}];
    rtrerrup=[rtrerrup;ML{84,ii}];
    oxtotcol=[oxtotcol;ML{89,ii}'];
    luftluft=[luftluft;ML{97,ii}];
end

%reconstruct the lower partial column values
priorlow=lowercon.*(0.2095./oxcon);
map=(((lsmap-1).*priorlow).*(oxcon./0.2095)+lowercon).*(0.2095./oxcon).*1e9;
maperr=lowerr.*priorlow.*1e9;

low_partcol=map; %MAP retrieved lower partial column DMF values
low_partcol_err=maperr;%Error for MAP retrieved lower partial column DMF values

uppercol=colmol;
uppercolox=colox;
priorhigh=uppercol.*(0.2095./uppercolox);    
highmap=usmap.*priorhigh.*1e9;
highmaperr=uperr.*priorhigh.*1e9;

high_partcol=highmap; %MAP retrieved upper partial column DMF values
high_partcol_err=highmaperr;%Error for MAP retrieved upper partial column DMF values

%remove cell structure to save space
clear ML

%save the calculated values to the file name specified
save(outputfn);

end


%}