

function [dtw,low_partcol,high_partcol,low_partcol_err,high_partcol_err]=...
    TARDISS_nc_CO2(etl,outputfn,zobs,localtime)

% function TARDISS_nc_CO2
% function to infer partial column values from the total column DMF values
% reported by TCCON in the private.nc files

% Input:
% etl - matlab structure containing the variables and data in the
% private.nc files reported by TCCON. The files can be reduced to include
% only the necessary variables from the private.nc file.
%
% outputfn - your chosen output file name as a string. 'output_etl' for
% example.
%
% zobs - observational elevation of the TCCON site in km. 
%
% localtime - standard time offset from UTC time. This is needed to align
% measurements to be within the same numerical day to avoid fitting values
% between days. 

% Output:
% ML - structure containing all the fit values described at the bottom of
% the script when they are assigned. ML is normally not saved in order to
% save space. Add to the output variables and comment line 607 to save.
%
% dtw - datenum vector for time series
% 
% low_partcol - lower partial column DMF values in ppm
% 
% high_partcol - upper partial column DMF values in ppm
% 
% low_partcol_err - lower partial column DMF error values in ppm
% 
% high_partcol_err - upper partial column DMF error values in ppm
% 
%
% Collated vectors of retrieved values are described at the end of the
% script and contain the values from the matrices in ML in vector format
% for ease of plotting. 

flag=etl.flag; % assign qc flag vector

% Operational fitting parameters - do not change unless testing 
% wg=1; % unused wg parameter
sc=1;
xp=2;

zminall=etl.zmin; %assign larger zmin vector

mavtime=etl.prior_time; %assign prior times
mavdtv=datevec(mavtime-(localtime/24)); %adjust time to local standard time

o2vsf=etl.vsf_o2; %assign O2 VSFs

%load in averaging kernels for the CO2 windows
load('co2akmats_TARDISS.mat')

%interpolating the aks to sza degrees
akbin=[10:10:90];
akbinint=[10:90];

for ii=1:51
    akv6220(ii,:)=interp1([10:10:90],coo6220akmat(ii,:),[10:90]);
    akv6339(ii,:)=interp1([10:10:90],coo6339akmat(ii,:),[10:90]);
    akv6073(ii,:)=interp1([10:10:90],coo6073akmat(ii,:),[10:90]);
    akv4852(ii,:)=interp1([10:10:90],coo4852akmat(ii,:),[10:90]);
end

coo6220akmat=akv6220;
coo6339akmat=akv6339;
coo6073akmat=akv6073;
coo4852akmat=akv4852;

%change to pst and get unique days 
etl.utc=etl.time;
etl.dtw=etl.time-(localtime/24);
days=unique(floor(etl.dtw));
days=days(days>0);

%round sza to nearest 1 degree
szabin=round(etl.solzen);
diffzen=etl.solzen-szabin;
 ll=length(szabin);
bin=[min(szabin):1:max(szabin)];
count=0;
for ii=bin
    count=count+1;
    etl.bin(szabin==ii,1)=count;
end

dayleng=length(days); %get number of unique days

zfilt=(flag==0);% QC flag filter

jjcount=0; %start count for days of measurement

for jj=1:dayleng
    
    jjcount=jjcount+1;
    
    clear aaa aaaa

    mask=(floor(etl.dtw)==days(jj))&(zfilt==1); %create filter for the day and the QC filter
    if sum(mask)==0
        jjcount=jjcount-1;
        continue
    end
    
    %assign daily values for variables and align a priori profiles
    asza=etl.solzen(mask);
    daynum=sum(mask);

    datw=etl.dtw(mask);
    datv=datevec(datw);
    dayun=unique(datv(:,1:3),'rows');
    lldatw=length(datw);
    
    zmin=zminall(mask);
    hite=etl.ak_altitude;
    luft=etl.xluft(mask);
    
    mavmask=sum(mavdtv(:,1:3)==dayun,2)==3;
    
    mavday=mavdtv(mavmask,:);
    mavdatw=datenum(mavday);
    llmavdatw=length(mavdatw);

    bb=length(bin);
    %find the nearest prior profile to the current point in the iteration
    for ii=1:llmavdatw
        aaa(:,ii)=(datw-mavdatw(ii)).*24;
        aaaa(:,ii)=abs(aaa(:,ii))<1.50;
    end
    
    %assign matmav values
    coap = etl.prior_1co2(:,mavmask);
    denap = etl.prior_density(:,mavmask);
    pres = etl.prior_pressure(:,mavmask);
    hiteapdt = (hite'.*ones(lldatw,51));
    oxygen=etl.prior_0o2(:,mavmask);
    water=etl.prior_1h2o(:,mavmask);
    coapdt=zeros(lldatw,51);
    presapdt=zeros(lldatw,51);
    denapdt=zeros(lldatw,51);
    oxygenapdt=zeros(lldatw,51);
    waterapdt=zeros(lldatw,51);
    
   % determine the partial column split point and the ratio of the levels
   % given the partial column split for integration
    lowerlim=nanmin(find(hite(:,1)-zobs>0));
    upperlim=nanmin(find(hite(:,1)-zobs-2>0));
    upperpathratio=((zobs+2)-hite(upperlim-1,1))./...
        (hite(upperlim,1)-hite(upperlim-1,1));
    
    
    for zz=1:llmavdatw
        for kk=1:lldatw
            if aaaa(kk,zz)==1  
                coapdt(kk,:)=coap(:,zz);
                presapdt(kk,:)=pres(:,zz);
                denapdt(kk,:)=denap(:,zz);
                oxygenapdt(kk,:)=oxygen(:,zz); 
                waterapdt(kk,:)=water(:,zz);
            end  
        end
    end
    
    % calculating the oxygen total column data
    oxygenapdt=(oxygenapdt.*o2vsf(mask));
        
    % calculate original vertical columns of species and oxygen
    [~,co_ovc,~]=vert_int(denapdt,hiteapdt,presapdt,coapdt,5,zmin,lowerlim,zobs);
    [~,ox_ovc,~]=vert_int(denapdt,hiteapdt,presapdt,oxygenapdt,5,zmin,lowerlim,zobs);
    
    coovc=co_ovc'./(ox_ovc./0.2095)';
    
    % calculate the median total column scalar 
    apscaleyy=nanmedian([etl.vsw_ada_xco2_6220(mask),...
        etl.vsw_ada_xco2_6339(mask),etl.vsw_ada_xwco2_6073(mask),...
        etl.vsw_ada_xlco2_4852(mask)],2);
  
    apscale=apscaleyy./coovc;
        
    %Make the jacobian values - termed k values for K matrix in the paper
    
    J6220=zeros(lldatw,51);
    J6339=J6220;
    J6073=J6220;
    J4852=J6220;
    
    daybin=etl.bin(mask);
    
     for kk=1:daynum
        J6220(kk,:)=coo6220akmat(:,daybin(kk))';
        J6339(kk,:)=coo6339akmat(:,daybin(kk))';
        J6073(kk,:)=coo6073akmat(:,daybin(kk))';
        J4852(kk,:)=coo4852akmat(:,daybin(kk))';
     end
    
     coapdtfull=coapdt.*apscale;
   
        %recreating the scaled ovcs
    coapdtfull6220=coapdtfull;
    coapdtfull6339=coapdtfull;
    coapdtfull6073=coapdtfull;
    coapdtfull4852=coapdtfull;

    %assign multipliers alone since retrieving column values
    %%
    
        %standard window weightings
    jcoo6220=J6220;
    jcoo6339=J6339;
    jcoo6073=J6073;
    jcoo4852=J4852;

    %assign multipliers with background prior
        
    jcoo6220full=jcoo6220.*coapdtfull6220;
    jcoo6339full=jcoo6339.*coapdtfull6339;
    jcoo6073full=jcoo6073.*coapdtfull6073;
    jcoo4852full=jcoo4852.*coapdtfull4852;
        
    %integrate to get the jacobian values
    [calparcolox,calccolox,calparcoloxup]=...
        vert_int(denapdt,hiteapdt,presapdt,oxygenapdt,5,zmin,lowerlim,zobs);
    [calparcolj6220,~,calparcolupj6220]=...
        vert_int(denapdt,hiteapdt,presapdt,jcoo6220full,5,zmin,lowerlim,zobs);
    [calparcolj6339,~,calparcolupj6339]=...
        vert_int(denapdt,hiteapdt,presapdt,jcoo6339full,5,zmin,lowerlim,zobs);
    [calparcolj6073,~,calparcolupj6073]=...
        vert_int(denapdt,hiteapdt,presapdt,jcoo6073full,5,zmin,lowerlim,zobs);
    [calparcolj4852,~,calparcolupj4852]=...
        vert_int(denapdt,hiteapdt,presapdt,jcoo4852full,5,zmin,lowerlim,zobs);

    upperlowerrat=calparcoloxup./calparcolox;
    ulr=upperlowerrat;

    JLa6220=calparcolj6220.*(0.2095./calccolox');
    JLa6339=calparcolj6339.*(0.2095./calccolox');
    JLa6073=calparcolj6073.*(0.2095./calccolox');
    JLa4852=calparcolj4852.*(0.2095./calccolox');
    
    JU6220=calparcolupj6220.*(0.2095./calccolox');
    JU6339=calparcolupj6339.*(0.2095./calccolox');
    JU6073=calparcolupj6073.*(0.2095./calccolox');
    JU4852=calparcolupj4852.*(0.2095./calccolox');

    %create the jacobian matrix
     Gw=[diag(JLa6220), diag(JU6220);...
        diag(JLa6339), diag(JU6339);...
        diag(JLa6073), diag(JU6073);...
        diag(JLa4852), diag(JU4852)];  
    
        if sum(sum(isnan(Gw)))~=0
            jjcount=jjcount-1;
            continue
        end
    %set number of windows for multipliers
    numwin=4;
   
    %create Y vector, do in situ correction, and calculate error values
    
    coo6220=etl.vsw_ada_xco2_6220(mask).*(1./1.0101);
    coo6339=etl.vsw_ada_xco2_6339(mask).*(1./1.0101);
    coo6073=etl.vsw_ada_xwco2_6073(mask).*(1./1.0008);
    coo4852=etl.vsw_ada_xlco2_4852(mask).*(1./1.0014);
    
    cooerr6220=(etl.vsw_co2_6220_error(mask)./etl.vsw_co2_6220(mask)).*coo6220;
    cooerr6339=(etl.vsw_co2_6339_error(mask)./etl.vsw_co2_6339(mask)).*coo6339;
    cooerr6073=(etl.vsw_wco2_6073_error(mask)./etl.vsw_wco2_6073(mask)).*coo6073;
    cooerr4852=(etl.vsw_lco2_4852_error(mask)./etl.vsw_lco2_4852(mask)).*coo4852;
    
    %Order matters for the alignment of the vectors
    yy=[coo6220;coo6339;coo6073;coo4852];
    yyerr=[cooerr6220;cooerr6339;cooerr6073;cooerr4852];
    
    oxscalevec=repmat(calccolox',numwin,1)./0.2095;
    
    [calcparcol,altovc6220,calcparcolup]=...
        vert_int(denapdt,hiteapdt,presapdt,coapdtfull6220,5,zmin,lowerlim,zobs);
    [~,altovc6339,~]=...
        vert_int(denapdt,hiteapdt,presapdt,coapdtfull6339,5,zmin,lowerlim,zobs);
    [~,altovc6073,~]=...
        vert_int(denapdt,hiteapdt,presapdt,coapdtfull6073,5,zmin,lowerlim,zobs);
    [~,altovc4852,~]=...
        vert_int(denapdt,hiteapdt,presapdt,coapdtfull4852,5,zmin,lowerlim,zobs);
 
    ovcvec=[altovc6220';altovc6339';altovc6073';altovc4852']./oxscalevec;
    
    lovc=length(altovc6220);
    aicf_vec=(1./1.0101).*ones(lovc,1);
    aicf_wco2_vec=(1./1.0008).*ones(lovc,1);
    aicf_lco2_vec=(1./1.0014).*ones(lovc,1);
    aicf_tot_vec=[aicf_vec;aicf_vec;aicf_wco2_vec;aicf_lco2_vec];
    
    xaovc=ovcvec;

    apscalevec=repmat(apscale,numwin,1);
    
    %create Se, model covariance matrix
    W=diag(yyerr).^2;
   %%
   % set the prior covariance matrix scaling, sc=1 is the operational value
   if sc==1
    % % five
    nn=1e-5;
    mm=1e-5;
    
   elseif sc==2
    % % five five
    nn=5e-5;
    mm=5e-5;
    
   elseif sc==3
    % % four
    nn=1e-4;
    mm=1e-4;
   end
    
    %create Sa
    Sa=[diag(mm.*ones(daynum,1)),diag(zeros(daynum,1));...
        diag(zeros(daynum,1)),diag(nn.*ones(daynum,1))];

    %create off-diagonals for Sa - comment for no temporal component
   %%{
    lim=floor(daynum)/3; %original , increasing 3 shortens covariance time
    for indi=1:lim
        for ind = 1:daynum-indi
            Sa(daynum+ind,daynum+ind+indi)=nn.*exp(-(indi+1)/(lim/2));
            Sa(daynum+ind+indi,daynum+ind)=nn.*exp(-(indi+1)/(lim/2));
        end
    end
     %%}
    
    %reassign values to match Rodgers/Connor
    Se=W;
    K=Gw;

    %create vectors and matrices for calculation
 d = (yy-xaovc);
 G=K;
 RG=rank(G);
 full=RG==2*daynum;

    %calculating the least squares solution and errors for least squares
    %solution
    
 ml2=inv(G'*G)*G'*d; %least squares solution
 
  Fx=G*ml2;
  
  nel=length(Fx)./4;
  RSS=zeros(nel,1);
  for ii=1:(nel)
      RSS(ii,1)=((Fx(ii,1)-d(ii,1)).^2)+((Fx(ii+nel,1)-d(ii+nel,1)).^2)+...
          ((Fx(ii+2*nel,1)-d(ii+2*nel,1)).^2)+((Fx(ii+3*nel,1)-d(ii+3*nel,1)).^2);
  end
  RSS=[RSS;RSS];

  
  Cov=inv(G'*G);
  Var=RSS.*Cov;
  ml2err=sqrt(diag(Var)); %least squares errors

 y=(K*(ml2))+xaovc; %result of forward model with least squares solution - unused
    
    %%
    %assigning the a priori partial column scalar values
 if xp==1
 % % % median of least squares solution as a priori 
 xa=[1.*nanmedian(ml2(1:daynum,1)).*ones(daynum,1);...
     1.*nanmedian(ml2(daynum+1:end,1)).*ones(daynum,1)];
 
 elseif xp==2
 % % least squares as a priori - use this one for operational co2 retrievals
  xa=[1.*(ml2(1:daynum,1));ml2(daynum+1:end,1)];

 elseif xp==3
% % static a priori scalar of one
 xa=[zeros(daynum,1);zeros(daynum,1)];
 end

%calculate the maximum a posteriori solution
 map=xa+Sa*G'*inv(G*Sa*G'+Se)*(d-G*xa);
 
 ymap=(K*(map))+xaovc; %result of forward model with MAP solution - unused
 
 %calculate fisher information matrix
 fishinfo=G'*inv(Se)*G+inv(Sa);
 %calculate averaging kernel
 Amat=inv(G'*inv(Se)*G+inv(Sa))*G'*inv(Se)*G;
 %calculate degrees of freedom in the fit
 ds=trace(Amat); %total DoF

 dslow=trace(Amat(1:daynum,1:daynum)); % lower partial column DoF
 dshigh=trace(Amat(daynum+1:end,daynum+1:end));% upper partial column DoF
  
 % calculate shannon information matrix
 H=-0.5.*log(det(eye(length(Amat))-Amat));

 % calculate covariance matrix
 covmat=inv(fishinfo);
 % calculate error of MAP retrieval
 stderr=zeros(length(Amat),1);
 for sstd=1:length(Amat)
    stderr(sstd)=sqrt(abs(covmat(sstd,sstd)));
 end
% calculate the retrieval noise and smoothing error
 s_hat=inv(G'*inv(Se)*G+inv(Sa));
 part=inv(G'*inv(Se)*G+inv(Sa));
Ssp=part'*inv(Sa)*part;
Srp=part'*G'*inv(Se)*G*part;
Ss=diag(Ssp);
Sr=diag(Srp);
 
% check for unphysical retrieval values
sealter=inv(Se.^0.5);
saalter=Sa.^0.5;
Khat=sealter*K*saalter;
    if sum(sum(isnan(Khat)))>0
        jjcount=jjcount-1;
        continue
    end
[Uk,Sk,Vk] = svd(Khat);
minkhat=min(size(Khat));
for ii=1:minkhat
sdiag(ii,1)=Sk(ii,ii);
end
sdiaggtone=sum(sdiag>1);

% Calculate further characterization data
calccol=calcparcolup;
 [calcparcol_ox,totoxcol,calccol_ox,oxycol]=...
     vert_int(denapdt,hiteapdt,presapdt,oxygenapdt,5,zmin,lowerlim,zobs);
[~,~,~,watercol]=...
    vert_int(denapdt,hiteapdt,presapdt,waterapdt,5,zmin,lowerlim,zobs);
 
watercol=watercol.*(0.2095./oxycol);


    %calculate gain matrix and vertical sensitivity values
gain=inv(G'*inv(Se)*G+inv(Sa))*G'*inv(Se);
    
jmat=[jcoo6220;jcoo6339;jcoo6073;jcoo4852];

scalarprofile= gain*(jmat).*([coapdtfull.*apscale;coapdtfull.*apscale]);

lowak=(scalarprofile(1:daynum,:));
highak=(scalarprofile(daynum+1:end,:));

%assign values from each day to cell structure - indexes not organized ina
%particular meaning
ML{1,jj}=ml2(1:daynum)+1; % least squares lower partial column scalar
ML{2,jj}=ml2(daynum+1:end)+1;% least squares upper partial column scalar
ML{3,jj}=calcparcol; % a priori species lower partial column value
ML{4,jj}=datw; % datenum time values for the day
ML{5,jj}=full; % specification if the retrieval is full rank - unused
ML{7,jj}=yy; % measurement vector for fit
ML{8,jj}=y; % forward model solution from least squares solution
ML{9,jj}=map(1:daynum)+1;% MAP lower partial column scalar
ML{10,jj}=map(daynum+1:end)+1;% MAP upper partial column scalar
ML{11,jj}=ymap;% forward model solution from MAP solution
ML{14,jj}=calcparcolup;% a priori species upper partial column value
ML{15,jj}=calcparcol_ox;% a priori O2 lower partial column value
ML{17,jj}=H;% Shannon information content
ML{19,jj}=fishinfo;% Fisher information matrix
ML{24,jj}=daynum;% number of measurements in a day
ML{25,jj}=calcparcolup;% a priori O2 upper partial column value
ML{26,jj}=calccol_ox;% a priori O2 total column value
ML{27,jj}=etl.solzen(mask);% solar zenith angles for the day
ML{29,jj}=etl.tout(mask);% ground temperature measured by TCCON
ML{30,jj}=stderr(1:daynum);% MAP lower partial column error values
ML{31,jj}=stderr(daynum+1:end);% MAP upper partial column error values
ML{34,jj}=G; % gain matrix
ML{35,jj}=Sa;% a priori covariance matrix
ML{36,jj}=Se;% model covariance matrix
ML{37,jj}=Amat;% temporal averaging kernel matrix
ML{38,jj}=dslow;% lower partial column daily DoFs
ML{39,jj}=dshigh;% upper partial column daily DoFs
ML{41,jj}=xa;% prior partial column scalar values used in the fit
ML{42,jj}=ml2;% least squares lower and upper partial column scalar
ML{43,jj}=ds;% all daily partial column daily DoFs
ML{44,jj}=jcoo6220;% averaging kernel values for 6220 cm-1 window
ML{45,jj}=jcoo6339;% averaging kernel values for 6339 cm-1 window
ML{46,jj}=jcoo6073;% averaging kernel values for 6073 cm-1 window
ML{47,jj}=jcoo4852;% averaging kernel values for 4852 cm-1 window
ML{49,jj}=apscalevec;% median VSF for TCCON windows
ML{50,jj}=coapdtfull;% a priori CO profiles
ML{53,jj}=d;% measurement vector for fit

%error reporting
ML{80,jj}=s_hat;
ML{81,jj}=Ss(1:daynum);% lower partial column smoothing error
ML{82,jj}=Sr(1:daynum);% lower partial column retrieval noise
ML{83,jj}=Ss(daynum+1:end);% upper partial column smoothing error
ML{84,jj}=Sr(daynum+1:end);% upper partial column retrieval noise
ML{86,jj}=etl.hout(mask);% relative humidity at surface of TCCON site 
ML{87,jj}=etl.pout(mask);% preasure measured at surface of TCCON site

ML{89,jj}=totoxcol';% total oxygen column value in molec/cm^2

ML{97,jj}=etl.xluft(mask); %xluft daily vector


end

%% creating full vectors from the daily vectors from the retrieval

[~,dayleng]=(size(ML));

%create empty space for values
ls=[];% least squares lower partial column scalar
us=[];% least squares upper partial column scalar
lowercon=[];% a priori species lower partial column value
uppercon=[];% a priori species upper partial column value
RG=[];% specification if the retrieval is full rank - unused
lsmap=[];% MAP lower partial column scalar
usmap=[];% MAP upper partial column scalar
dtw=[];% datenum time values for the day
tccon=[];% measurement vector for fit
mappost=[];% forward model solution from MAP solution
mlpost=[];% forward model solution from least squares solution
oxcon=[];% a priori O2 lower partial column value
shannon=[];% Shannon information content
ptsinday=[];% number of measurements in a day
colox=[];% a priori O2 total column value
colmol=[];% a priori O2 upper partial column value
solzen=[];% solar zenith angles for the day
temperature=[];% ground temperature measured by TCCON
lowerr=[];% MAP lower partial column error values
uperr=[];% MAP upper partial column error values
dsl=[];% lower partial column daily DoFs
dsh=[];% upper partial column daily DoFs
totalds=[];% total daily partial column daily DoFs
smterr=[];% lower partial column smoothing error
rtrerr=[];% lower partial column retrieval noise
smterrup=[];% upper partial column smoothing error
rtrerrup=[];% upper partial column retrieval noise
rh=[];% relative humidity at surface of TCCON site 
pres=[];% preasure measured at surface of TCCON site
oxtotcol=[];% total oxygen column value in molec/cm^2
luftluft=[];%xluft vector


%iterate over the retrieval days
for ii= 1:dayleng
    
    ls=[ls;ML{1,ii}];
    us=[us;ML{2,ii}];
    lowercon=[lowercon;ML{3,ii}];
    RG=[RG;ML{5,ii}];
    lsmap=[lsmap;ML{9,ii}];
    usmap=[usmap;ML{10,ii}];
    dtw=[dtw;ML{4,ii}];
    tccon=[tccon;ML{7,ii}];
    mappost=[mappost;ML{11,ii}];
    mlpost=[mlpost;ML{8,ii}];
    uppercon=[uppercon;ML{14,ii}];
    oxcon=[oxcon;ML{15,ii}];
    if ~isinf(ML{17,ii})
        shannon=[shannon;ML{17,ii}];
    else
        shannon=[shannon;zeros(length(ML{17,ii}))];
    end
    ptsinday=[ptsinday;ML{24,ii}];
    colmol=[colmol;ML{25,ii}];
    colox=[colox;ML{26,ii}];
    solzen=[solzen;ML{27,ii}];
    temperature=[temperature;ML{29,ii}];
    lowerr=[lowerr;ML{30,ii}];
    uperr=[uperr;ML{31,ii}];
    dsl=[dsl;ML{38,ii}];
    dsh=[dsh;ML{39,ii}];
    totalds=[totalds;ML{43,ii}];
    smterr=[smterr;ML{81,ii}];
    rtrerr=[rtrerr;ML{82,ii}];
    smterrup=[smterrup;ML{83,ii}];
    rtrerrup=[rtrerrup;ML{84,ii}];
    rh=[rh;ML{86,ii}];
    pres=[pres;ML{87,ii}];
    oxtotcol=[oxtotcol;ML{89,ii}];
    luftluft=[luftluft;ML{97,ii}];
end

%reconstruct the lower partial column values
priorlow=lowercon.*(0.2095./oxcon);
map=(((lsmap-1).*priorlow).*(oxcon./0.2095)+lowercon).*(0.2095./oxcon).*1e6;
maperr=lowerr.*priorlow.*1e6;

low_partcol=map; %MAP retrieved lower partial column DMF values
low_partcol_err=maperr;%Error for MAP retrieved lower partial column DMF values

uppercol=colmol;
uppercolox=colox;
priorhigh=uppercol.*(0.2095./uppercolox);    
highmap=usmap.*priorhigh.*1e6;
highmaperr=uperr.*priorhigh.*1e6;

high_partcol=highmap; %MAP retrieved upper partial column DMF values
high_partcol_err=highmaperr;%Error for MAP retrieved upper partial column DMF values

%remove cell structure to save space
clear ML

%save the calculated values to the file name specified
save(outputfn);

end

%}